@@ -1,6 +1,6 @@
 #' Vuong Tests for Model Comparison
 #'
-#' Test pairs of non-nested models using Vuong's (1989) theory.  This includes
+#' Test pairs of models using Vuong's (1989) theory.  This includes
 #' a test of model distinguishability and a test of model fit.
 #'
 #' For non-nested models, the test of distinguishability indicates whether or
@@ -8,6 +8,10 @@
 #' data.  The LRT then indicates whether or not one model fits better
 #' than another.
 #'
+#' For nested models (\code{nested=TRUE}), both tests serve as robust
+#' alternatives to the classical likelihood ratio tests.  In this case,
+#' the \code{adj} argument is ignored.
+#'
 #' Users should take care to ensure that the two models have
 #' the same dependent variable (or, for lavaan objects, identical
 #' modeled variables), with observations ordered identically within
@@ -18,6 +22,8 @@
 #'
 #' @param object1 a model object
 #' @param object2 a model object
+#' @param nested if \code{TRUE}, models are assumed to be nested
+#' @param adj Should an adjusted test statistic be calculated?  Defaults to \dQuote{none}, with possible adjustments being \dQuote{aic} and \dQuote{bic}
 #'
 #' @author Ed Merkle and Dongjun You
 #'
@@ -38,7 +44,7 @@
 #' house3 <- glm(Freq ~ Infl, family=poisson, data=housing)
 #' ## house3 is nested within house1 and house2
 #' anova(house3, house1, test="Chisq")
-#' anova(house2, house1, test="Chisq")
+#' anova(house3, house2, test="Chisq")
 #'
 #' ## house 2 is not nested in house1, so this test is invalid
 #' anova(house2, house1, test="Chisq")
@@ -70,7 +76,7 @@
 #' @importFrom sandwich estfun
 #' @importFrom CompQuadForm imhof
 #' @export
-vuongtest <- function(object1, object2) {
+vuongtest <- function(object1, object2, nested=FALSE, adj="none") {
   classA <- class(object1)[1L]
   classB <- class(object2)[1L]
   callA <- if (isS4(object1)) object1@call else object1$call
@@ -79,46 +85,75 @@ vuongtest <- function(object1, object2) {
   llA <- llcont(object1)
   llB <- llcont(object2)
 
-  ## if (!isTRUE(all.equal(sum(llA), as.numeric(logLik(object1)))))
-  ##   stop("The individual log-likelihoods do not sum up to the log-likelihood. Please report your model and object to the maintainer.")
-
-  ## if (!isTRUE(all.equal(sum(llB), as.numeric(logLik(object2)))))
-  ##   stop("The individual log-likelihoods do not sum up to the log-likelihood. Please report your model and object to the maintainer.")
+  ## If nested==TRUE, decide which is the full model by seeing
+  ## which has the larger log-likelihood.  From here on,
+  ## object1 is the full model
+  if(nested){
+      if(sum(llB) > sum(llA)){
+          tmp <- object1
+          object1 <- object2
+          object2 <- tmp
+          tmp <- llA
+          llA <- llB
+          llB <- tmp
+      }
+  }
 
   ## Eq (4.2)
   n <- NROW(llA)
   omega.hat.2 <- (n-1)/n * var(llA - llB)
 
   ## Get p-value of weighted chi-square dist
-  ## Need to install the dr package
-  lamstar2 <- calcLambda(object1, object2, n)
+  lamstar <- calcLambda(object1, object2, n)
+  ## Note: dr package requires non-negative weights, which
+  ##       does not help when nested==TRUE
   ## tmp <- dr.pvalue(lamstar2, n * omega.hat.2)
   ## pOmega <- tmp[[4]]
-  ## Alternative:
-  ## library(CompQuadForm)
-  pOmega <- imhof(n * omega.hat.2, lamstar2)[[1]]
+  pOmega <- imhof(n * omega.hat.2, lamstar^2)[[1]]
 
   ## Calculate and test LRT; Eq (6.4)
   lr <- sum(llA - llB)
   teststat <- (1/sqrt(n)) * lr/sqrt(omega.hat.2)
 
-  ## Two 1-tailed p-values:
-  pLRTA <- pnorm(teststat, lower.tail=FALSE)
-  pLRTB <- pnorm(teststat)
+  ## Adjustments to test statistics
+  if(adj=="aic"){
+    teststat <- teststat - (length(coef(object1)) - length(coef(object2)))
+  }
+  if(adj=="bic"){
+    teststat <- teststat -
+      (length(coef(object1)) - length(coef(object2))) * log(n)/2
+  }
 
-  ## Interval for BIC differences
-  #bicA <- BIC(object1)
-  #bicB <- BIC(object2)
-  #bicdiff <- bicA - bicB
+  ## Null distribution and test stat depend on nested
+  if(nested){
+      teststat <- 2 * lr
+      ## lamstar is negative due to the ordering of objects in calcLambda();
+      ## we could get the same thing via calcLambda(object2, object1, n)
+      pLRTA <- imhof(teststat, -lamstar)[[1]]
 
-  ## BIC CI
-  #ci <- bicdiff + qnorm(c(alpha/2,(1-alpha/2)))*sqrt(n * 4 * omega.hat.2)
+      ## Compare different approximations
+      ## Empirical
+      ## tmp <- matrix(rchisq(1000 * length(lamstar), 1), 1000, length(lamstar))
+      ## edist <- apply(tmp, 1, function(x) sum(lamstar*x))
+      ## print(summary(edist))
+      ## imhof without negative weights
+      ## print(imhof(teststat, lamstar)[[1]])
+      ## davies without negative weights
+      ## print(davies(teststat, lamstar)$Qq)
+
+      pLRTB <- NA
+  } else {
+      ## Two 1-tailed p-values from a normal:
+      pLRTA <- pnorm(teststat, lower.tail=FALSE)
+      pLRTB <- pnorm(teststat)
+  }
 
   rval <- list(omega = omega.hat.2, p_omega = pOmega,
-               z_LRT = teststat,
+               LRTstat = teststat,
                p_LRT = list(A=pLRTA, B=pLRTB),
                class = list(class1=classA, class2=classB),
-               call = list(call1=callA, call2=callB))#, BICCI=ci)
+               call = list(call1=callA, call2=callB),
+               nested = nested)
   class(rval) <- "vuongtest"
   return(rval)
 }
@@ -160,8 +195,8 @@ calcLambda <- function(object1, object2, n) {
 
   lamstar <- eigen(W, only.values=TRUE)$values
   ## Discard imaginary part, as it only occurs for tiny eigenvalues?
-  ## using Mod
-  Mod(lamstar)^2
+  ## using Re
+  Re(lamstar)
 }
 
 ################################################################
@@ -172,28 +207,37 @@ calcLambda <- function(object1, object2, n) {
 print.vuongtest <- function(x, ...) {
   cat("\nModel 1 \n")
   cat(" Class:", x$class$class1, "\n")
-  cat(" Call:", deparse(x$call$call1), "\n", fill=TRUE)
+  cat(" Call:", deparse(x$call$call1, nlines=1), "\n\n")
   cat("Model 2 \n")
-  cat(" Class:", x$class$class2, "\n\n")
-  cat(" Call:", deparse(x$call$call2), "\n\n", fill=TRUE)
+  cat(" Class:", x$class$class2, "\n")
+  cat(" Call:", deparse(x$call$call2, nlines=1), "\n\n\n")
 
   cat("Variance test \n")
-  cat("  H0: Model 1 and Model 2 are equivalent for the focal population", "\n")
+  cat("  H0: Model 1 and Model 2 are indistinguishable", "\n")
+  cat("  H1: Model 1 and Model 2 are distinguishable", "\n")
   cat("  w2 = ", formatC(x$omega, digits=3L, format="f"), ",   ",
       "p = ", format.pval(x$p_omega, digits=3L), "\n\n", sep="")
 
-  cat("Non-nested likelihood ratio test \n")
-  cat("  H0: Model fits are equal for the focal population \n")
-  cat("  H1A: Model 1 fits better than Model 2 \n")
-  cat("  z = ", formatC(x$z_LRT, digits=3L, format="f"), ",   ",
-      "p = ", format.pval(x$p_LRT[[1]], digits=3L), "\n", sep="")
-  cat("  H1B: Model 2 fits better than Model 1 \n")
-  cat("  z = ", formatC(x$z_LRT, digits=3L, format="f"), ",   ",
-      "p = ", format.pval(x$p_LRT[[2]], digits=4L), "\n", sep="")
+  if(x$nested){
+      cat("Robust likelihood ratio test of distinguishable models \n")
+      cat("  H0: Model 2 fits as well as Model 1 \n")
+      cat("  H1: Model 1 fits better than Model 2 \n")
+      cat("  LR = ", formatC(x$LRTstat, digits=3L, format="f"), ",   ",
+          "p = ", format.pval(x$p_LRT[[1]], digits=3L), "\n", sep="")
+  } else {
+      cat("Non-nested likelihood ratio test \n")
+      cat("  H0: Model fits are equal for the focal population \n")
+      cat("  H1A: Model 1 fits better than Model 2 \n")
+      cat("  z = ", formatC(x$LRTstat, digits=3L, format="f"), ",   ",
+          "p = ", format.pval(x$p_LRT[[1]], digits=3L), "\n", sep="")
+      cat("  H1B: Model 2 fits better than Model 1 \n")
+      cat("  z = ", formatC(x$LRTstat, digits=3L, format="f"), ",   ",
+          "p = ", format.pval(x$p_LRT[[2]], digits=4L), "\n", sep="")
+  }
 }
 
 
 .onAttach <- function(...) {
-  packageStartupMessage("  This is nonnest2 0.1-1
+  packageStartupMessage("  This is nonnest2 0.2
   nonnest2 is BETA software! Please report any bugs.")
 }
